@@ -67,15 +67,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r21 = _mm256_loadu_ps(r2 + 8);
                 __m256 _r22 = _mm256_loadu_ps(r2 + 16);
 
-                _sum0 = _mm256_fmadd_ps(_k00, _r00, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k01, _r01, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k02, _r02, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k10, _r10, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k11, _r11, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k12, _r12, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k20, _r20, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k21, _r21, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k22, _r22, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k00, _r00, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k01, _r01, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k02, _r02, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k10, _r10, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k11, _r11, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k12, _r12, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k20, _r20, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k21, _r21, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k22, _r22, _sum0);
 
                 __m256 _sum1 = _bias0;
                 __m256 _r03 = _mm256_loadu_ps(r0 + 24);
@@ -83,15 +83,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r23 = _mm256_loadu_ps(r2 + 24);
                 _mm256_storeu_ps(outptr0, _sum0);
 
-                _sum1 = _mm256_fmadd_ps(_k00, _r01, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k01, _r02, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k02, _r03, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k10, _r11, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k11, _r12, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k12, _r13, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k20, _r21, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k21, _r22, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k22, _r23, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k00, _r01, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k01, _r02, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k02, _r03, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k10, _r11, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k11, _r12, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k12, _r13, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k20, _r21, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k21, _r22, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k22, _r23, _sum1);
 
                 __m256 _sum2 = _bias0;
                 __m256 _r04 = _mm256_loadu_ps(r0 + 32);
@@ -99,15 +99,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r24 = _mm256_loadu_ps(r2 + 32);
                 _mm256_storeu_ps(outptr0 + 8, _sum1);
 
-                _sum2 = _mm256_fmadd_ps(_k00, _r02, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k01, _r03, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k02, _r04, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k10, _r12, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k11, _r13, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k12, _r14, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k20, _r22, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k21, _r23, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k22, _r24, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k00, _r02, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k01, _r03, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k02, _r04, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k10, _r12, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k11, _r13, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k12, _r14, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k20, _r22, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k21, _r23, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k22, _r24, _sum2);
 
                 __m256 _sum3 = _bias0;
                 __m256 _r05 = _mm256_loadu_ps(r0 + 40);
@@ -115,15 +115,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r25 = _mm256_loadu_ps(r2 + 40);
                 _mm256_storeu_ps(outptr0 + 16, _sum2);
 
-                _sum3 = _mm256_fmadd_ps(_k00, _r03, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k01, _r04, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k02, _r05, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k10, _r13, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k11, _r14, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k12, _r15, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k20, _r23, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k21, _r24, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k22, _r25, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k00, _r03, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k01, _r04, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k02, _r05, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k10, _r13, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k11, _r14, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k12, _r15, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k20, _r23, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k21, _r24, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k22, _r25, _sum3);
 
                 __m256 _sum4 = _bias0;
                 __m256 _r06 = _mm256_loadu_ps(r0 + 48);
@@ -131,15 +131,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r26 = _mm256_loadu_ps(r2 + 48);
                 _mm256_storeu_ps(outptr0 + 24, _sum3);
 
-                _sum4 = _mm256_fmadd_ps(_k00, _r04, _sum4);
-                _sum4 = _mm256_fmadd_ps(_k01, _r05, _sum4);
-                _sum4 = _mm256_fmadd_ps(_k02, _r06, _sum4);
-                _sum4 = _mm256_fmadd_ps(_k10, _r14, _sum4);
-                _sum4 = _mm256_fmadd_ps(_k11, _r15, _sum4);
-                _sum4 = _mm256_fmadd_ps(_k12, _r16, _sum4);
-                _sum4 = _mm256_fmadd_ps(_k20, _r24, _sum4);
-                _sum4 = _mm256_fmadd_ps(_k21, _r25, _sum4);
-                _sum4 = _mm256_fmadd_ps(_k22, _r26, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k00, _r04, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k01, _r05, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k02, _r06, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k10, _r14, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k11, _r15, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k12, _r16, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k20, _r24, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k21, _r25, _sum4);
+                _sum4 = _mm256_comp_fmadd_ps(_k22, _r26, _sum4);
 
                 __m256 _sum5 = _bias0;
                 __m256 _r07 = _mm256_loadu_ps(r0 + 56);
@@ -147,15 +147,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r27 = _mm256_loadu_ps(r2 + 56);
                 _mm256_storeu_ps(outptr0 + 32, _sum4);
 
-                _sum5 = _mm256_fmadd_ps(_k00, _r05, _sum5);
-                _sum5 = _mm256_fmadd_ps(_k01, _r06, _sum5);
-                _sum5 = _mm256_fmadd_ps(_k02, _r07, _sum5);
-                _sum5 = _mm256_fmadd_ps(_k10, _r15, _sum5);
-                _sum5 = _mm256_fmadd_ps(_k11, _r16, _sum5);
-                _sum5 = _mm256_fmadd_ps(_k12, _r17, _sum5);
-                _sum5 = _mm256_fmadd_ps(_k20, _r25, _sum5);
-                _sum5 = _mm256_fmadd_ps(_k21, _r26, _sum5);
-                _sum5 = _mm256_fmadd_ps(_k22, _r27, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k00, _r05, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k01, _r06, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k02, _r07, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k10, _r15, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k11, _r16, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k12, _r17, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k20, _r25, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k21, _r26, _sum5);
+                _sum5 = _mm256_comp_fmadd_ps(_k22, _r27, _sum5);
 
                 __m256 _sum6 = _bias0;
                 __m256 _r08 = _mm256_loadu_ps(r0 + 64);
@@ -163,15 +163,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r28 = _mm256_loadu_ps(r2 + 64);
                 _mm256_storeu_ps(outptr0 + 40, _sum5);
 
-                _sum6 = _mm256_fmadd_ps(_k00, _r06, _sum6);
-                _sum6 = _mm256_fmadd_ps(_k01, _r07, _sum6);
-                _sum6 = _mm256_fmadd_ps(_k02, _r08, _sum6);
-                _sum6 = _mm256_fmadd_ps(_k10, _r16, _sum6);
-                _sum6 = _mm256_fmadd_ps(_k11, _r17, _sum6);
-                _sum6 = _mm256_fmadd_ps(_k12, _r18, _sum6);
-                _sum6 = _mm256_fmadd_ps(_k20, _r26, _sum6);
-                _sum6 = _mm256_fmadd_ps(_k21, _r27, _sum6);
-                _sum6 = _mm256_fmadd_ps(_k22, _r28, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k00, _r06, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k01, _r07, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k02, _r08, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k10, _r16, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k11, _r17, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k12, _r18, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k20, _r26, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k21, _r27, _sum6);
+                _sum6 = _mm256_comp_fmadd_ps(_k22, _r28, _sum6);
 
                 __m256 _sum7 = _bias0;
                 __m256 _r09 = _mm256_loadu_ps(r0 + 72);
@@ -179,15 +179,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r29 = _mm256_loadu_ps(r2 + 72);
                 _mm256_storeu_ps(outptr0 + 48, _sum6);
 
-                _sum7 = _mm256_fmadd_ps(_k00, _r07, _sum7);
-                _sum7 = _mm256_fmadd_ps(_k01, _r08, _sum7);
-                _sum7 = _mm256_fmadd_ps(_k02, _r09, _sum7);
-                _sum7 = _mm256_fmadd_ps(_k10, _r17, _sum7);
-                _sum7 = _mm256_fmadd_ps(_k11, _r18, _sum7);
-                _sum7 = _mm256_fmadd_ps(_k12, _r19, _sum7);
-                _sum7 = _mm256_fmadd_ps(_k20, _r27, _sum7);
-                _sum7 = _mm256_fmadd_ps(_k21, _r28, _sum7);
-                _sum7 = _mm256_fmadd_ps(_k22, _r29, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k00, _r07, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k01, _r08, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k02, _r09, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k10, _r17, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k11, _r18, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k12, _r19, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k20, _r27, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k21, _r28, _sum7);
+                _sum7 = _mm256_comp_fmadd_ps(_k22, _r29, _sum7);
                 _mm256_storeu_ps(outptr0 + 56, _sum7);
 
                 r0 += 64;
@@ -209,15 +209,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r21 = _mm256_loadu_ps(r2 + 8);
                 __m256 _r22 = _mm256_loadu_ps(r2 + 16);
 
-                _sum0 = _mm256_fmadd_ps(_k00, _r00, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k01, _r01, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k02, _r02, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k10, _r10, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k11, _r11, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k12, _r12, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k20, _r20, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k21, _r21, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k22, _r22, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k00, _r00, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k01, _r01, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k02, _r02, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k10, _r10, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k11, _r11, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k12, _r12, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k20, _r20, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k21, _r21, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k22, _r22, _sum0);
 
                 __m256 _sum1 = _bias0;
                 __m256 _r03 = _mm256_loadu_ps(r0 + 24);
@@ -225,15 +225,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r23 = _mm256_loadu_ps(r2 + 24);
                 _mm256_storeu_ps(outptr0, _sum0);
 
-                _sum1 = _mm256_fmadd_ps(_k00, _r01, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k01, _r02, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k02, _r03, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k10, _r11, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k11, _r12, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k12, _r13, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k20, _r21, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k21, _r22, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k22, _r23, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k00, _r01, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k01, _r02, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k02, _r03, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k10, _r11, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k11, _r12, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k12, _r13, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k20, _r21, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k21, _r22, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k22, _r23, _sum1);
 
                 __m256 _sum2 = _bias0;
                 __m256 _r04 = _mm256_loadu_ps(r0 + 32);
@@ -241,15 +241,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r24 = _mm256_loadu_ps(r2 + 32);
                 _mm256_storeu_ps(outptr0 + 8, _sum1);
 
-                _sum2 = _mm256_fmadd_ps(_k00, _r02, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k01, _r03, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k02, _r04, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k10, _r12, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k11, _r13, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k12, _r14, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k20, _r22, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k21, _r23, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k22, _r24, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k00, _r02, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k01, _r03, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k02, _r04, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k10, _r12, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k11, _r13, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k12, _r14, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k20, _r22, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k21, _r23, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k22, _r24, _sum2);
 
                 __m256 _sum3 = _bias0;
                 __m256 _r05 = _mm256_loadu_ps(r0 + 40);
@@ -257,15 +257,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r25 = _mm256_loadu_ps(r2 + 40);
                 _mm256_storeu_ps(outptr0 + 16, _sum2);
 
-                _sum3 = _mm256_fmadd_ps(_k00, _r03, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k01, _r04, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k02, _r05, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k10, _r13, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k11, _r14, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k12, _r15, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k20, _r23, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k21, _r24, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k22, _r25, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k00, _r03, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k01, _r04, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k02, _r05, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k10, _r13, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k11, _r14, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k12, _r15, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k20, _r23, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k21, _r24, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k22, _r25, _sum3);
 
                 _mm256_storeu_ps(outptr0 + 24, _sum3);
 
@@ -288,15 +288,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r21 = _mm256_loadu_ps(r2 + 8);
                 __m256 _r22 = _mm256_loadu_ps(r2 + 16);
 
-                _sum0 = _mm256_fmadd_ps(_k00, _r00, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k01, _r01, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k02, _r02, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k10, _r10, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k11, _r11, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k12, _r12, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k20, _r20, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k21, _r21, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k22, _r22, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k00, _r00, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k01, _r01, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k02, _r02, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k10, _r10, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k11, _r11, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k12, _r12, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k20, _r20, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k21, _r21, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k22, _r22, _sum0);
 
                 __m256 _sum1 = _bias0;
                 __m256 _r03 = _mm256_loadu_ps(r0 + 24);
@@ -304,15 +304,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r23 = _mm256_loadu_ps(r2 + 24);
                 _mm256_storeu_ps(outptr0, _sum0);
 
-                _sum1 = _mm256_fmadd_ps(_k00, _r01, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k01, _r02, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k02, _r03, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k10, _r11, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k11, _r12, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k12, _r13, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k20, _r21, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k21, _r22, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k22, _r23, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k00, _r01, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k01, _r02, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k02, _r03, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k10, _r11, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k11, _r12, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k12, _r13, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k20, _r21, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k21, _r22, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k22, _r23, _sum1);
 
                 _mm256_storeu_ps(outptr0 + 8, _sum1);
 
@@ -335,15 +335,15 @@ static void convdw3x3s1_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r21 = _mm256_loadu_ps(r2 + 8);
                 __m256 _r22 = _mm256_loadu_ps(r2 + 16);
 
-                _sum0 = _mm256_fmadd_ps(_k00, _r00, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k01, _r01, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k02, _r02, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k10, _r10, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k11, _r11, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k12, _r12, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k20, _r20, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k21, _r21, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k22, _r22, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k00, _r00, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k01, _r01, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k02, _r02, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k10, _r10, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k11, _r11, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k12, _r12, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k20, _r20, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k21, _r21, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k22, _r22, _sum0);
 
                 _mm256_storeu_ps(outptr0, _sum0);
 
@@ -419,15 +419,15 @@ static void convdw3x3s2_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r21 = _mm256_loadu_ps(r2 + 8);
                 __m256 _r22 = _mm256_loadu_ps(r2 + 16);
 
-                _sum0 = _mm256_fmadd_ps(_k00, _r00, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k01, _r01, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k02, _r02, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k10, _r10, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k11, _r11, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k12, _r12, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k20, _r20, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k21, _r21, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k22, _r22, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k00, _r00, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k01, _r01, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k02, _r02, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k10, _r10, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k11, _r11, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k12, _r12, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k20, _r20, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k21, _r21, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k22, _r22, _sum0);
 
                 __m256 _sum1 = _bias0;
                 __m256 _r03 = _mm256_loadu_ps(r0 + 24);
@@ -438,15 +438,15 @@ static void convdw3x3s2_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r24 = _mm256_loadu_ps(r2 + 32);
                 _mm256_storeu_ps(outptr0, _sum0);
 
-                _sum1 = _mm256_fmadd_ps(_k00, _r02, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k01, _r03, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k02, _r04, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k10, _r12, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k11, _r13, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k12, _r14, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k20, _r22, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k21, _r23, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k22, _r24, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k00, _r02, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k01, _r03, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k02, _r04, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k10, _r12, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k11, _r13, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k12, _r14, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k20, _r22, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k21, _r23, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k22, _r24, _sum1);
 
                 __m256 _sum2 = _bias0;
                 __m256 _r05 = _mm256_loadu_ps(r0 + 40);
@@ -457,15 +457,15 @@ static void convdw3x3s2_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r26 = _mm256_loadu_ps(r2 + 48);
                 _mm256_storeu_ps(outptr0 + 8, _sum1);
 
-                _sum2 = _mm256_fmadd_ps(_k00, _r04, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k01, _r05, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k02, _r06, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k10, _r14, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k11, _r15, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k12, _r16, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k20, _r24, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k21, _r25, _sum2);
-                _sum2 = _mm256_fmadd_ps(_k22, _r26, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k00, _r04, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k01, _r05, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k02, _r06, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k10, _r14, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k11, _r15, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k12, _r16, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k20, _r24, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k21, _r25, _sum2);
+                _sum2 = _mm256_comp_fmadd_ps(_k22, _r26, _sum2);
 
                 __m256 _sum3 = _bias0;
                 __m256 _r07 = _mm256_loadu_ps(r0 + 56);
@@ -476,15 +476,15 @@ static void convdw3x3s2_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r28 = _mm256_loadu_ps(r2 + 64);
                 _mm256_storeu_ps(outptr0 + 16, _sum2);
 
-                _sum3 = _mm256_fmadd_ps(_k00, _r06, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k01, _r07, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k02, _r08, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k10, _r16, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k11, _r17, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k12, _r18, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k20, _r26, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k21, _r27, _sum3);
-                _sum3 = _mm256_fmadd_ps(_k22, _r28, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k00, _r06, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k01, _r07, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k02, _r08, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k10, _r16, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k11, _r17, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k12, _r18, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k20, _r26, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k21, _r27, _sum3);
+                _sum3 = _mm256_comp_fmadd_ps(_k22, _r28, _sum3);
 
                 _mm256_storeu_ps(outptr0 + 24, _sum3);
 
@@ -507,15 +507,15 @@ static void convdw3x3s2_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r21 = _mm256_loadu_ps(r2 + 8);
                 __m256 _r22 = _mm256_loadu_ps(r2 + 16);
 
-                _sum0 = _mm256_fmadd_ps(_k00, _r00, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k01, _r01, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k02, _r02, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k10, _r10, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k11, _r11, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k12, _r12, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k20, _r20, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k21, _r21, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k22, _r22, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k00, _r00, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k01, _r01, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k02, _r02, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k10, _r10, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k11, _r11, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k12, _r12, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k20, _r20, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k21, _r21, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k22, _r22, _sum0);
 
                 __m256 _sum1 = _bias0;
                 __m256 _r03 = _mm256_loadu_ps(r0 + 24);
@@ -526,15 +526,15 @@ static void convdw3x3s2_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r24 = _mm256_loadu_ps(r2 + 32);
                 _mm256_storeu_ps(outptr0, _sum0);
 
-                _sum1 = _mm256_fmadd_ps(_k00, _r02, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k01, _r03, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k02, _r04, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k10, _r12, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k11, _r13, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k12, _r14, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k20, _r22, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k21, _r23, _sum1);
-                _sum1 = _mm256_fmadd_ps(_k22, _r24, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k00, _r02, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k01, _r03, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k02, _r04, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k10, _r12, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k11, _r13, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k12, _r14, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k20, _r22, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k21, _r23, _sum1);
+                _sum1 = _mm256_comp_fmadd_ps(_k22, _r24, _sum1);
 
                 _mm256_storeu_ps(outptr0 + 8, _sum1);
 
@@ -557,15 +557,15 @@ static void convdw3x3s2_fp16_pack8_avx(const Mat& bottom_blob, Mat& top_blob, co
                 __m256 _r21 = _mm256_loadu_ps(r2 + 8);
                 __m256 _r22 = _mm256_loadu_ps(r2 + 16);
 
-                _sum0 = _mm256_fmadd_ps(_k00, _r00, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k01, _r01, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k02, _r02, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k10, _r10, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k11, _r11, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k12, _r12, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k20, _r20, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k21, _r21, _sum0);
-                _sum0 = _mm256_fmadd_ps(_k22, _r22, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k00, _r00, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k01, _r01, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k02, _r02, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k10, _r10, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k11, _r11, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k12, _r12, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k20, _r20, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k21, _r21, _sum0);
+                _sum0 = _mm256_comp_fmadd_ps(_k22, _r22, _sum0);
                 _mm256_storeu_ps(outptr0, _sum0);
                 r0 += 2 * 8;
                 r1 += 2 * 8;
